require "hydra"

class HydrangeaDataset < ActiveFedora::Base
  
  include Hydra::ModelMethods
  include Uva::CommonModelMethods
  
  has_relationship "parts", :is_part_of, :inbound => true
  
  # Uses the Hydra Rights Metadata Schema for tracking access permissions & copyright
  has_metadata :name => "rightsMetadata", :type => Hydra::RightsMetadata 
  
  # Uses the Hydra MODS Dataset profile for tracking most of the descriptive metadata
  has_metadata :name => "descMetadata", :type => Hydra::ModsDataset 

  # A place to put extra metadata values
  has_metadata :name => "properties", :type => ActiveFedora::MetadataDatastream do |m|
    m.field 'collection', :string
    m.field 'depositor', :string
    m.field 'released', :string
    m.field 'release_to', :string
    m.field 'deposit_rights', :string
    m.field 'not_confidential', :string
    m.field 'grant_requirements', :string
    m.field 'worldwide', :string
    m.field 'irb', :string
  end
  
  def self.solr_object_type_facet
     "Dataset"
  end
  
  def self.supports_multiple_files?
    true
  end
  
  def self.only_pdfs?
    false
  end
    
  # Tests whether the object is ready to release
  # @return true if ready, otherwise a Hash where response[:failures] is an array of failure messages
  def test_release_readiness
    author_entries = datastreams["descMetadata"].find_by_terms_and_value('//oxns:name[@type="personal" and contains(oxns:role, "dtc") and string-length(oxns:namePart[@type="family"]) > 0 ]')    
    title_values = datastreams["descMetadata"].term_values(:title_info, :main_title)
    data_created_values = datastreams["descMetadata"].term_values(:origin_info, :date_created)
    abstract_values = datastreams["descMetadata"].term_values(:abstract)
    release_to_values = datastreams["properties"].release_to_values
    deposit_rights_values = datastreams["properties"].deposit_rights_values
    not_confidential_values = datastreams["properties"].not_confidential_values
    grant_requirements_values = datastreams["properties"].grant_requirements_values
    worldwide_values = datastreams["properties"].worldwide_values
    irb_values = datastreams["properties"].irb_values
    data_collection_start_date_values = datastreams["descMetadata"].term_values(:data_collection_start_date)
    data_collection_end_date_values = datastreams["descMetadata"].term_values(:data_collection_end_date)

    # check for files
    files = self.file_objects({:response_format=>:solr, :rows=>UVA_SOLR_ROWS_NUM})
    has_readme = false
    has_dataset = false
    files.each do |file|
      type = file[solr_name("description", :text)].first rescue ""
      has_readme = true if type =~ /readme/i
      has_dataset = true if type =~ /dataset/i
    end
    response = {:failures=>[]}
    
    if author_entries.empty?
      response[:failures] << "Please provide an author (first name and last name)."
    end  
    
    if title_values.empty? || title_values.first.empty?
       response[:failures] << "Please provide a title."
     end  
    
    if data_created_values.empty? || data_created_values.first.empty?
      response[:failures] << "Please provide data created date."
    else
      Date.strptime(data_created_values.first, "%Y-%m-%d") rescue response[:failures] << "Date Files/Data Package Created is not a valid date"
    end
    
    unless data_collection_start_date_values.empty? || data_collection_start_date_values.first.empty?
      Date.strptime(data_collection_start_date_values.first, "%Y-%m-%d") rescue response[:failures] << "Data Collection Start Date is not a valid date"
    end
    
    unless data_collection_end_date_values.empty? || data_collection_end_date_values.first.empty?
      Date.strptime(data_collection_end_date_values.first, "%Y-%m-%d") rescue response[:failures] << "Data Collection End Date is not a valid date"
    end
    
    unless data_collection_start_date_values.empty? || data_collection_start_date_values.first.empty?
      unless data_collection_end_date_values.empty? || data_collection_end_date_values.first.empty?
        if Date.parse(data_collection_start_date_values.first) > Date.parse(data_collection_end_date_values.first)
        response[:failures] << "Data Collection Start Date should not be later than Data Collection End Date."
        end
      end    
    end
    
    if abstract_values.empty? || abstract_values.first.empty?
      response[:failures] << "Please provide an abstract."
    end
      
    if !has_readme
      response[:failures] << "Please upload a Readme file for your submission"
    end
    
    if !has_dataset
      response[:failures] << "Please upload a dataset file for your submission"
    end
    
    if release_to_values.empty? || !["uva", "public_dataset"].include?(release_to_values.first)
      response[:failures] << "You must agree to the U.Va. Libra Dataset Submission Agreement."
    end
    
    if deposit_rights_values.empty? || deposit_rights_values.first != "yes"
      response[:failures] << "You must agree that you have deposit rights"
    end
    
    if not_confidential_values.empty? || not_confidential_values.first != "yes"
      response[:failures] << "You must agree that data is not confidential"
    end
    
    if grant_requirements_values.empty? || grant_requirements_values.first != "yes"
      response[:failures] << "You must agree that grant requirements have been met"      
    end

    if worldwide_values.empty? || worldwide_values.first != "yes"
      response[:failures] << "You must agree to worldwide access"      
    end
    
    if irb_values.empty? || irb_values.first != "yes"
      response[:failures] << "You must certify IRB compliance"      
    end
    
        
    if response[:failures].empty? 
      return true
    else
      return response
    end
  
  end
  
end
