#!/usr/bin/env ruby -wKU
require "hydra"

class HydrangeaFourthYearThesis < ActiveFedora::Base
  
  include Hydra::ModelMethods
  include Uva::CommonModelMethods
  
  has_relationship "parts", :is_part_of, :inbound => true
  
  # Uses the Hydra Rights Metadata Schema for tracking access permissions & copyright
  has_metadata :name => "rightsMetadata", :type => Hydra::RightsMetadata 
  
  # Uses the Hydra MODS 4th year thesis profile for tracking most of the descriptive metadata
  has_metadata :name => "descMetadata", :type => Hydra::ModsFourthYearThesis 

  # A place to put extra metadata values
  has_metadata :name => "properties", :type => ActiveFedora::MetadataDatastream do |m|
    m.field 'collection', :string
    m.field 'depositor', :string
    m.field 'released', :string
    m.field 'release_to', :string
    m.field 'peer_reviewed', :string
  end
  
  def self.sis_mapping
    { self.libra_type =>  self.sis_type }
  end
  
  def self.libra_type
    'hydrangea_fourth_year_thesis'
  end
  
  def self.sis_type
    'FOURTHESISDEF'
  end
  
  def self.solr_object_type_facet
    'Fourth Year Thesis'
  end
  
  def non_editable_fields(computing_id)
    sis_non_editable_fields(computing_id)
  end
  
  def self.supports_multiple_files?
    true
  end
  
  # Tests whether the object is ready to release
  # @return true if ready, otherwise a Hash where response[:failures] is an array of failure messages
  def test_release_readiness
    author_entries = datastreams["descMetadata"].find_by_terms_and_value('//oxns:name[@type="personal" and contains(oxns:role, "aut") and string-length(oxns:namePart[@type="family"]) > 0 ]')    
    advisor_entries = datastreams["descMetadata"].find_by_terms_and_value('//oxns:name[@type="personal" and contains(oxns:role, "ths") and string-length(oxns:namePart[@type="family"]) > 0 ]')    
        
    title_values = datastreams["descMetadata"].term_values(:title_info, :main_title)
    approved_values = datastreams["descMetadata"].term_values(:origin_info, :date_issued)
    degree_values = datastreams["descMetadata"].term_values(:extension, :degree, :level)
    abstract_values = datastreams["descMetadata"].term_values(:abstract)
    release_to_values = datastreams["properties"].release_to_values
    embargo_months = datastreams["rightsMetadata"].term_values(:embargo, :embargo_release_date)[0]
    
    # check for files
    files = self.file_objects({:response_format=>:solr, :rows=>UVA_SOLR_ROWS_NUM})
    has_main_thesis = false
    main_thesis_count = 0
    files.each do |file|
      type = file[solr_name("description", :text)].first rescue ""
      if type =~ /thesis/i
        has_main_thesis = true 
        main_thesis_count += 1
       end
    end
    
    response = {:failures=>[]}

    if title_values.empty? || title_values.first.empty?
      response[:failures] << "Please provide a title."
    end  
    if author_entries.empty?
      response[:failures] << "Please provide an author (first name and last name)."
    end  
    if advisor_entries.empty?
      response[:failures] << "Please provide an advisor (first name and last name)."
    end
    if approved_values.empty?
      response[:failures] << "Please provide the approval date."
    end
    if degree_values.empty?
      response[:failures] << "Please provide the degree level."
    end
    if abstract_values.empty? || abstract_values.first.empty?
      response[:failures] << "Please provide an abstract."
    end
    if parts(:response_format=>:id_array).empty?
      response[:failures] << "Please attach at least one file."
    end
    if !has_main_thesis
      response[:failures] << "Please upload your main thesis file for your submission."
    end
    if main_thesis_count > 1
      response[:failures] << "Only one thesis file can be uploaded for your submission."
    end
    if release_to_values.empty? || !["uva", "uetd"].include?(release_to_values.first)
      response[:failures] << "You must agree to the U.Va. Libra Submission Agreement."
    end
    if embargo_months.nil?
      response[:failures] << "Please select embargo period."
    end
    
    if response[:failures].empty? 
      return true
    else
      return response
    end
  end
end
