module Hydra::ModelMethods
  require_dependency 'vendor/plugins/hydra_repository/lib/hydra/model_methods.rb'
  #
  # Adds metadata about the depositor to the asset 
  #
  def apply_depositor_metadata(depositor_id)
    prop_ds = self.datastreams_in_memory["properties"]
    rights_ds = self.datastreams_in_memory["rightsMetadata"]
  
    if !prop_ds.nil? && prop_ds.respond_to?(:depositor_values)
      prop_ds.depositor_values = depositor_id unless prop_ds.nil?
    end
    rights_ds.update_indexed_attributes([:edit_access, :person]=>depositor_id) unless rights_ds.nil?
    
    # apply values from SIS if we have them
    if self.class.respond_to?(:sis_mapping) and sis_record = SisRecord.find_by_computing_id_and_milestone_and_libra_approved(depositor_id, self.class.sis_type, nil)
      apply_sis_values(sis_record, 0)
    else
      apply_ldap_values(depositor_id, 0)
    end
    return true
  end
  
  #
  # looks through the params to fetch the computing id, then dispatches to ldap lookup to update
  #
  def update_from_computing_id(params)
    computing_id_field = fetch_field_from_params("computing_id", params)
    return if computing_id_field.empty?
    apply_ldap_values(computing_id_field[1], computing_id_field[0])
  end

  #
  # applies the sis attributes
  #
  def apply_sis_values(sis_record, person_number)
    desc_ds = self.datastreams_in_memory["descMetadata"]
    return if desc_ds.nil?
    desc_ds.find_by_terms(:person, :computing_id)[person_number].content = sis_record.computing_id
    desc_ds.find_by_terms(:person, :first_name)[person_number].content = sis_record.first_name
    desc_ds.find_by_terms(:person, :last_name)[person_number].content = sis_record.last_name
    desc_ds.find_by_terms(:person, :institution)[person_number].content = LDAP_INSTITUTION   
    desc_ds.find_by_terms(:person, :description)[person_number].content = sis_record.academic_plan
    desc_ds.find_by_terms(:mods, :title_info, :main_title)[0].content = sis_record.title_of_work
    desc_ds.find_by_terms(:extension, :degree, :level)[0].content = sis_record.plan_degree
    desc_ds.find_by_terms(:origin_info, :date_issued)[0].content = sis_record.date_approved
  end
  
  #
  # applies the ldap attributes
  #
  def apply_ldap_values(computing_id, person_number)
    person = Ldap::Person.new(computing_id)
    desc_ds = self.datastreams_in_memory["descMetadata"]
    return if desc_ds.nil?
    begin
      desc_ds.find_by_terms(:person, :computing_id)[person_number].content = person.computing_id
      desc_ds.find_by_terms(:person, :first_name)[person_number].content = person.first_name
      desc_ds.find_by_terms(:person, :last_name)[person_number].content = person.last_name
      desc_ds.find_by_terms(:person, :institution)[person_number].content = person.institution    
      desc_ds.find_by_terms(:person, :description)[person_number].content = person.department
    rescue
      return
    end
  end
  
  #
  # find the given field within the params and returns the field number and value
  # for that field
  #
  def fetch_field_from_params(field, params)
    params["asset"].each_pair do |datastream_name,fields|
      if params.fetch("field_selectors",false) && params["field_selectors"].fetch(datastream_name, false)
        fields.each_pair do |field_name,field_values|
          if field_name =~ /#{field}/
            field_number = field_name[/_\d+_/].tr("_", "").to_i
            field_value = field_values["0"]
            return [field_number, field_value]
          end
        end
      end
    end
    return []
  end
  
  #
  # do ISSN lookup
  #
  def issn_lookup(params)
    issn = fetch_field_from_params("issn", params)[1] rescue ""
    return if issn.blank?
    desc_ds = self.datastreams_in_memory["descMetadata"]
    return if desc_ds.nil?
    issn_data = IssnLookup.new(issn)
    desc_ds.find_by_terms(:journal, :title_info, :main_title)[0].content = issn_data.journal_title
    desc_ds.find_by_terms(:journal, :origin_info, :publisher)[0].content = issn_data.publisher
  end

  def update_from_embargo_selection(params)
    return unless params[:asset] and params[:asset][:sisMetadata]
    selection = params[:asset][:sisMetadata][":embargo_selection"]["0"]     
    if params[:id].nil? || params[:id].blank? || SisRecord.find_by_libra_id(params[:id]).nil?
      sis_record = SisRecord.find_by_computing_id_and_milestone_and_libra_approved(self.depositor, self.class.sis_type, nil)
    else
      sis_record = SisRecord.find_by_libra_id(params[:id])
    end   

    return if sis_record.blank?
    sis_record.embargo_months = selection
    sis_record.save        
    selection == "0" ? val = Date.today.strftime('%Y-%m-%d') : val = (Date.today + selection.to_i.months).strftime('%Y-%m-%d')
    vals = {[:embargo, :embargo_release_date]=>{"0"=>val}}
    datastream = {:datastreams=>"rightsMetadata"}     
    result=self.update_indexed_attributes(vals, datastream) 
    self.save           
    return result
  end

  #
  # determines if an item has been released for circulation
  #
  def is_released?
    self.datastreams["properties"].released_values[0] == "true" rescue false
  end
  
  def released_to
    self.datastreams["properties"].release_to_values[0] rescue ""
  end

  def depositor
    self.datastreams["properties"].depositor_values[0] rescue ""
  end

  
end
