module Uva::CommonModelMethods
  
  def to_solr(solr_doc=Solr::Document.new,opts={})
    super(solr_doc, opts)
    if submitted_for_release?
      solr_doc = apply_release(solr_doc)
    end
    solr_doc
  end
  
  # Placeholder method that applies access rights if the object has been released and satisfies the requirements for release
  # The value of the "release_to" entry in the properties datastream is used to decide who should be granted access to the book.
  # If no release_to value has been set, defaults to "public"
  #
  # Requirements for release
  # * title is set
  # * an author is set
  # * at least one file has been uploaded
  #
  def apply_release(solr_doc)
    if ready_to_release?

      released = solr_doc.instance_variable_get(:@fields).find { |f| f.name == "released_facet" }
      released.value = "true"
      
      solr_doc << Solr::Field.new(:released_dt => Time.now.strftime('%Y-%m-%dT%H:%M:%SZ') )

      # The order of the release_to checks below is critical. They proceed from most permissive to most restrictive.
      release_to = datastreams["properties"].release_to_values.first
      if release_to == "public"
        solr_doc << Solr::Field.new(:read_access_group_t => release_to)
      elsif release_to == "uetd"
        solr_doc << Solr::Field.new(:read_access_group_t => release_to)
      elsif release_to == "public_dataset"
        solr_doc << Solr::Field.new(:read_access_group_t => release_to)
      else
        solr_doc << Solr::Field.new(:read_access_group_t => 'uva')
      end
      set_sis_fields(solr_doc)
    end
    solr_doc
  end
  
  def set_sis_fields(solr_doc)
    if self.class.respond_to?(:sis_mapping) and sis_record = SisRecord.find_by_computing_id_and_milestone(solr_doc[:depositor_t], self.class.sis_type)
      if sis_record.libra_approved.blank?
        sis_record.libra_approved = Date.today
        sis_record.libra_id = solr_doc[:id]
        sis_record.save
      end
    end    
  end
  
  def dataset?
    self.class == HydrangeaDataset
  end

  # Tests whether the object has been submitted for release yet.
  # @return [Boolean]
  def submitted_for_release?
    datastreams["properties"].released_values == ["true"]
  end
  
  # Returns true/false based on the results of test_release_readiness
  # @return [Boolean]
  def ready_to_release?
    if test_release_readiness == true
      return true
    else
      return false
    end
  end
  
  def can_use_public_license?
    return false if ["hydrangea_fourth_year_thesis", "hydrangea_masters_thesis", "hydrangea_doctoral_dissertation"].include?(datastreams["properties"].collection_values[0])
    return true
  end
  
  def can_have_more_authors?
    return false if ["hydrangea_fourth_year_thesis", "hydrangea_masters_thesis", "hydrangea_doctoral_dissertation"].include?(datastreams["properties"].collection_values[0])
    return true
  end

  def sis_non_editable_fields(computing_id)
    if !non_sis_etd?(computing_id)
      [[{:person=>0}, :institution], [{:person=>0}, :description], [:extension, :degree, :level], [:origin_info, :date_issued], [:mods, :title_info, :main_title]]
    else
      [[{:person=>0}, :institution]]
    end
  end

  def non_sis_etd?(computing_id)
    if self.class.respond_to?(:sis_mapping) and sis_record = SisRecord.find_by_computing_id_and_milestone(computing_id, self.class.sis_type)
      if sis_record.empl_id.match(/^[A-Z]*[0-9]*/).nil?
        return false
      else
        return true
      end
    end

  end

  def languages
    {
      "afr" => "Afrikaans",
      "amh" => "Amharic",
      "ara" => "Arabic",
      "arm" => "Armenian",
      "aze" => "Azerbaijani",
      "baq" => "Basque",
      "bel" => "Belarusian",
      "bos" => "Bosnian",
      "bul" => "Bulgarian",
      "bur" => "Burmese",
      "chi" => "Chinese",
      "cze" => "Czech",
      "dan" => "Danish",
      "dut" => "Dutch; Flemish",
      "eng" => "English",
      "fin" => "Finnish",
      "fre" => "French",
      "geo" => "Georgian",
      "ger" => "German",
      "gle" => "Irish",
      "gre" => "Greek",
      "heb" => "Hebrew",
      "hin" => "Hindi",
      "hrv" => "Croatian",
      "hun" => "Hungarian",
      "ice" => "Icelandic",
      "ind" => "Indonesian",
      "ita" => "Italian",
      "jpn" => "Japanese",
      "kir" => "Kirghiz; Kyrgyz",
      "kor" => "Korean",
      "kur" => "Kurdish",
      "lao" => "Lao",
      "lat" => "Latin",
      "lav" => "Latvian",
      "lit" => "Lithuanian",
      "nor" => "Norwegian",
      "per" => "Persian",
      "pol" => "Polish",
      "por" => "Portuguese",
      "rum" => "Romanian",
      "rus" => "Russian",
      "slo" => "Slovak",
      "slv" => "Slovenian",
      "spa" => "Spanish",
      "srp" => "Serbian",
      "swe" => "Swedish",
      "tgl" => "Tagalog",
      "tha" => "Thai",
      "tib" => "Tibetan",
      "tur" => "Turkish",
      "ukr" => "Ukrainian",
      "uzb" => "Uzbek",
      "vie" => "Vietnamese",
      "wel" => "Welsh",
      "yid" => "Yiddish"
    }
  end
  
  
end
