module Uva::CommonTermMethods

  def self.included(base)
    base.send :extend,  StaticMethods
    base.send :include, InstanceMethods
    base.send :include, Hydra::CommonModsIndexMethods
    base.send :include, Uva::ModsIndexMethods
  end
  
  module StaticMethods
    def set_basics(t)
      set_root(t)
      set_title(t)
      set_language(t)
      set_abstract_subject_and_topic(t)
      set_name(t)
    end
  
    def set_root(t)
      t.root(:path=>"mods", 
       "xmlns:xlink"=>"http://www.w3.org/1999/xlink",
         "xmlns:xsi"=>"http://www.w3.org/2001/XMLSchema-instance",
         "xmlns"=>"http://www.loc.gov/mods/v3",
         "xmlns:etd" => "http://www.ntltd.org/standards/metadata/etdms/1.0/etdms.xsd",
         "xsi:schemaLocation"=>"http://www.loc.gov/mods/v3 http://www.loc.gov/standards/mods/v3/mods-3-3.xsd")
    end
  
    def set_title(t)
      t.title_info(:path=>"titleInfo") {
        t.main_title(:path=>"title", :label=>"title")
        t.language(:index_as=>[:facetable],:path=>{:attribute=>"lang"})
      }
    end
  
    def set_language(t)
      t.language{
        t.lang_code(:index_as=>[:facetable], :path=>"languageTerm", :attributes=>{:type=>"code"})
      }
    end
  
    def set_abstract_subject_and_topic(t)
      t.abstract   
      t.subject {
        t.topic(:index_as=>[:facetable])
      }      
      t.topic_tag(:proxy=>[:subject, :topic])    
    end
  
    def set_name(t)
      # This is a mods:name.  The underscore is purely to avoid namespace conflicts.
      t.name_ {
        # this is a namepart
        t.namePart(:type=>:string, :label=>"generic name")
        # affiliations are great
        t.affiliation
        t.institution(:path=>"affiliation", :index_as=>[:facetable], :label=>"organization")
        t.displayForm
        t.role(:ref=>[:role])
        t.description(:index_as=>[:facetable])
        t.date(:path=>"namePart", :attributes=>{:type=>"date"})
        t.last_name(:path=>"namePart", :attributes=>{:type=>"family"})
        t.first_name(:path=>"namePart", :attributes=>{:type=>"given"}, :label=>"first name")
        t.terms_of_address(:path=>"namePart", :attributes=>{:type=>"termsOfAddress"})
        t.computing_id(:path=>"namePart", :attributes=>{:type=>:none})        
      }
      # lookup :person, :first_name        
      t.person(:ref=>:name, :attributes=>{:type=>"personal"}, :index_as=>[:facetable])
      t.department(:proxy=>[:person,:description],:index_as=>[:facetable])
      #t.organization(:ref=>:name, :attributes=>{:type=>"corporate"}, :index_as=>[:facetable])
      t.conference(:ref=>:name, :attributes=>{:type=>"conference"}, :index_as=>[:facetable])
      t.role {
        t.text(:path=>"roleTerm",:attributes=>{:type=>"text"})
        t.code(:path=>"roleTerm",:attributes=>{:type=>"code"})
      }
    end
    
    def set_sponsor(t)
      t.sponsor(:ref=>:name, :attributes=>{:type=>"corporate"}, :index_as=>[:facetable]) {
        t.sponsor_name(:path=>"namePart")
      }
    end
    
    def build_template(format)
      builder = Nokogiri::XML::Builder.new do |xml|
        xml.mods(:version=>"3.3", "xmlns:xlink"=>"http://www.w3.org/1999/xlink",
          "xmlns:xsi"=>"http://www.w3.org/2001/XMLSchema-instance",
          "xmlns"=>"http://www.loc.gov/mods/v3",
          "xmlns:etd" => "http://www.ntltd.org/standards/metadata/etdms/1.0/etdms.xsd",
          "xsi:schemaLocation"=>"http://www.loc.gov/mods/v3 http://www.loc.gov/standards/mods/v3/mods-3-3.xsd") {
            
            ### title
            xml.titleInfo(:lang=>"") {
              xml.title
            }
            
            ### name
            xml.name(:type=>"personal") {
              xml.namePart(:type=>"given")
              xml.namePart(:type=>"family")
              xml.affiliation
              xml.namePart
              xml.description
              xml.role {
                if [:dataset].include?(format)
                  xml.roleTerm("dtc", :authority=>"marcrelator", :type=>"code")
                  xml.roleTerm("Data contributor", :authority=>"marcrelator", :type=>"text")                                                         
                else
                  xml.roleTerm("aut", :authority=>"marcrelator", :type=>"code")
                  xml.roleTerm("author", :authority=>"marcrelator", :type=>"text")
                end
              }
            }
            
            ### sponsoring institution
            unless [:fourth_year_thesis, :masters_thesis, :doctoral_dissertation, :dataset].include?(format)
              xml.name(:type=>"corporate", :authority=>"lcnaf") {
                xml.namePart
                xml.role {
                  xml.roleTerm("Sponsor", :authority=>"marcrelator", :type=>"text")
                }
              }
              xml.typeOfResource("text")
            end
            
            ### Dataset type of resource
            if [:dataset].include?(format)
              xml.typeOfResource("software, multimedia")
              xml.physicalDescription {
                xml.form("electronic", :authority=>"marcform")
                xml.digitalOrigin("born digital")
                xml.internetMediaType("text/xml")
              }
            end
                        
            ### ETD advisor
            if [:fourth_year_thesis, :masters_thesis, :doctoral_dissertation].include?(format)
              xml.name(:type=>"personal") {
                xml.namePart(:type=>"given")
                xml.namePart(:type=>"family")
                xml.affiliation
                xml.namePart
                xml.description
                xml.role {
                  xml.roleTerm("ths", :authority=>"marcrelator", :type=>"code")
                  xml.roleTerm("advisor", :authority=>"marcrelator", :type=>"text")
                }
              }
              
              ### ETD degree
              # the underscore is to avoid naming conflicts
              xml.extension_ {
                xml['etd'].degree {
                  if [:fourth_year_thesis].include?(format)
                    xml['etd'].name("Bachelor's thesis")
                  elsif [:masters_thesis].include?(format)
                    xml['etd'].name("Master's thesis")
                  else
                    xml['etd'].name("Doctoral dissertation")
                  end
                  xml['etd'].level
                  xml['etd'].discipline
                }
              }
            end  ### end ETD-specifics
            
            ### genre - articles, books, conference papers
            if [:article, :book, :conference_paper].include?(format)
              xml.genre(:authority=>"marcgt")  {
                xml.text("article") if format == :article
                xml.text("book") if format == :book
                xml.text("conference publication") if format == :conference_paper
              }
            end
             
             
            ### genre - book chapters 
            if [:book_chapter, :book_part].include?(format)
              xml.genre(:authority=>"aat")  {
                xml.text("chapters (layout features)")
              }
            end
             
            ### genre - article preprints
            if format == :article_preprint
              xml.genre(:authority=>"local") {
                xml.text("preprint")
              }
            end
            
            ### language
            unless [:dataset].include?(format)
              xml.language {
                xml.languageTerm(:authority=>"iso639-2b", :type=>"code")
              }
            end
            
            ### abstract, subject 
            xml.abstract
            xml.subject
            
             
            ### books - ISBN, publisher, date
            if format == :book
              xml.identifier(:type=>"isbn")
              xml.originInfo {
                xml.publisher
                xml.place {
                  xml.placeTerm
                }
                xml.dateIssued(:keydate=>"yes", :encoding=>"iso8601")
                xml.dateOther
                xml.issuance(:authority=>"local")
              }
            end             
             
            if [:conference_paper, :article, :article_preprint, :book_chapter, :book_part].include?(format)
              xml.relatedItem(:type=>"host") {
               
                # conference papers - conference name, date
                if format == :conference_paper
                  xml.name(:type=>"conference") {
                    xml.namePart
                    xml.namePart(:type=>"date", :encoding=>"iso8601")
                    xml.namePart(:type=>"date")
                    xml.affiliation
                  }
                end
               
                # articles, article preprints, and books - journal title, book name
                if [:article, :article_preprint, :book_chapter, :book_part].include?(format)
                  xml.titleInfo {
                    xml.title
                  }
                  xml.identifier(:type=>"issn") if format == :article
                  xml.identifier(:type=>"isbn") if [:book_chapter, :book_part].include?(format)
                   
                  if [:book_part].include?(format)
                    xml.name(:type=>"personal") {
                      xml.namePart(:type=>"given")
                      xml.namePart(:type=>"family")
                      xml.role {
                        xml.roleTerm("edt", :authority=>"marcrelator", :type=>"code")
                        xml.roleTerm("editor", :authority=>"marcrelator", :type=>"text")
                      }
                    }
                  end
               
                  xml.originInfo {
                    xml.publisher
                    if [:book_chapter, :book_part].include?(format)
                      xml.place {
                        xml.placeTerm(:type=>"text")
                      }
                    end
                    xml.dateIssued(:encoding=>"iso8601") if [:article, :article_preprint].include?(format)
                    xml.dateIssued(:encoding=>"iso8601", :keyDate=>"yes") if [:book_chapter, :book_part].include?(format)
                    xml.dateOther if [:book_chapter, :book_part].include?(format)
                    xml.issuance(:authority=>"local") if [:article, :article_preprint, :book_chapter, :book_part].include?(format)
                  }
                  xml.part {
                    if[:article, :article_preprint?].include?(format)
                      xml.detail(:type=>"volume") {
                        xml.number
                      }
                      xml.detail(:type=>"number") {
                        xml.number
                      }
                    end
                    xml.extent(:unit=>"pages") {
                      xml.start
                      xml.end
                    }
                    xml.date(:keydate=>"yes", :encoding=>"iso8601") if [:article, :article_preprint].include?(format)
                    xml.date if [:article, :article_preprint].include?(format)
                  }
                end  
              }
            end
            
            if [:fourth_year_thesis, :masters_thesis, :doctoral_dissertation].include?(format)
              xml.relatedItem(:type=>"host") {
                xml.originInfo {
                  xml.dateIssued(:encoding=>"iso8601")
                  xml.dateOther
                }
              }
            end
             
            if [:book, :article, :article_preprint, :book_chapter, :book_part].include?(format)
              xml.relatedItem(:type=>"otherVersion") {
                xml.location {
                  xml.url
                }
              }             
            end
          }
      end
      return builder.doc
    end
    
    def build_person(format)
      builder = Nokogiri::XML::Builder.new do |xml|
        xml.name(:type=>"personal") {
          xml.namePart(:type=>"given")
          xml.namePart(:type=>"family")
          xml.affiliation
          xml.namePart
          xml.description
          xml.role {
            if format == :dataset
              xml.roleTerm("Data contributor", :type=>"text")
              xml.roleTerm("dtc", :type=>"code")              
            else
              xml.roleTerm("author", :type=>"text")
              xml.roleTerm("aut", :type=>"code")
            end
          }
        }
      end
      return builder.doc.root
    end  
    
    def build_advisor
      builder = Nokogiri::XML::Builder.new do |xml|
        xml.name(:type=>"personal") {
          xml.namePart(:type=>"given")
          xml.namePart(:type=>"family")
          xml.affiliation
          xml.namePart
          xml.description
          xml.role {
            xml.roleTerm("ths", :type=>"code")
            xml.roleTerm("advisor", :type=>"text")
          }
        }
      end
      return builder.doc.root
    end
    
    def build_editor
      builder = Nokogiri::XML::Builder.new do |xml|
        xml.name(:type=>"personal") {
          xml.namePart(:type=>"given")
          xml.namePart(:type=>"family")
          xml.role {
            xml.roleTerm("edt", :authority=>"marcrelator", :type=>"code")
            xml.roleTerm("editor", :authority=>"marcrelator", :type=>"text")
          }
        }
      end
      return builder.doc.root
    end
  
    def build_full_name
      builder = Nokogiri::XML::Builder.new do |xml|
        xml.full_name(:type => "personal")
      end
      return builder.doc.root
    end
  
    def build_organization
      builder = Nokogiri::XML::Builder.new do |xml|
        xml.name(:type=>"corporate") {
          xml.namePart
          xml.role {
            xml.roleTerm(:authority=>"marcrelator", :type=>"text")
          }                          
        }
      end
      return builder.doc.root
    end
  
    def build_conference
      builder = Nokogiri::XML::Builder.new do |xml|
        xml.name(:type=>"conference") {
          xml.namePart
          xml.role {
            xml.roleTerm(:authority=>"marcrelator", :type=>"text")
          }                          
        }
      end
      return builder.doc.root
    end
        
    def common_relator_terms
       {"aut" => "Author",
        "clb" => "Collaborator",
        "com" => "Compiler",
        "ctb" => "Contributor",
        "cre" => "Creator",
        "edt" => "Editor",
        "ill" => "Illustrator",
        "oth" => "Other",
        "trl" => "Translator",
        }
    end

    def person_relator_terms
      {"aut" => "Author",
       "clb" => "Collaborator",
       "com" => "Compiler",
       "cre" => "Creator",
       "ctb" => "Contributor",
       "edt" => "Editor",
       "ill" => "Illustrator",
       "res" => "Researcher",
       "rth" => "Research team head",
       "rtm" => "Research team member",
       "trl" => "Translator"
       }
    end

    def conference_relator_terms
      {
        "hst" => "Host"
      }
    end

    def organization_relator_terms
      {
        "fnd" => "Funder",
        "hst" => "Host"
      }
    end

    def dc_relator_terms
       {"acp" => "Art copyist",
        "act" => "Actor",
        "adp" => "Adapter",
        "aft" => "Author of afterword, colophon, etc.",
        "anl" => "Analyst",
        "anm" => "Animator",
        "ann" => "Annotator",
        "ant" => "Bibliographic antecedent",
        "app" => "Applicant",
        "aqt" => "Author in quotations or text abstracts",
        "arc" => "Architect",
        "ard" => "Artistic director ",
        "arr" => "Arranger",
        "art" => "Artist",
        "asg" => "Assignee",
        "asn" => "Associated name",
        "att" => "Attributed name",
        "auc" => "Auctioneer",
        "aud" => "Author of dialog",
        "aui" => "Author of introduction",
        "aus" => "Author of screenplay",
        "aut" => "Author",
        "bdd" => "Binding designer",
        "bjd" => "Bookjacket designer",
        "bkd" => "Book designer",
        "bkp" => "Book producer",
        "bnd" => "Binder",
        "bpd" => "Bookplate designer",
        "bsl" => "Bookseller",
        "ccp" => "Conceptor",
        "chr" => "Choreographer",
        "clb" => "Collaborator",
        "cli" => "Client",
        "cll" => "Calligrapher",
        "clt" => "Collotyper",
        "cmm" => "Commentator",
        "cmp" => "Composer",
        "cmt" => "Compositor",
        "cng" => "Cinematographer",
        "cnd" => "Conductor",
        "cns" => "Censor",
        "coe" => "Contestant -appellee",
        "col" => "Collector",
        "com" => "Compiler",
        "cos" => "Contestant",
        "cot" => "Contestant -appellant",
        "cov" => "Cover designer",
        "cpc" => "Copyright claimant",
        "cpe" => "Complainant-appellee",
        "cph" => "Copyright holder",
        "cpl" => "Complainant",
        "cpt" => "Complainant-appellant",
        "cre" => "Creator",
        "crp" => "Correspondent",
        "crr" => "Corrector",
        "csl" => "Consultant",
        "csp" => "Consultant to a project",
        "cst" => "Costume designer",
        "ctb" => "Contributor",
        "cte" => "Contestee-appellee",
        "ctg" => "Cartographer",
        "ctr" => "Contractor",
        "cts" => "Contestee",
        "ctt" => "Contestee-appellant",
        "cur" => "Curator",
        "cwt" => "Commentator for written text",
        "dfd" => "Defendant",
        "dfe" => "Defendant-appellee",
        "dft" => "Defendant-appellant",
        "dgg" => "Degree grantor",
        "dis" => "Dissertant",
        "dln" => "Delineator",
        "dnc" => "Dancer",
        "dnr" => "Donor",
        "dpc" => "Depicted",
        "dpt" => "Depositor",
        "drm" => "Draftsman",
        "drt" => "Director",
        "dsr" => "Designer",
        "dst" => "Distributor",
        "dtc" => "Data contributor ",
        "dte" => "Dedicatee",
        "dtm" => "Data manager ",
        "dto" => "Dedicator",
        "dub" => "Dubious author",
        "edt" => "Editor",
        "egr" => "Engraver",
        "elg" => "Electrician ",
        "elt" => "Electrotyper",
        "eng" => "Engineer",
        "etr" => "Etcher",
        "exp" => "Expert",
        "fac" => "Facsimilist",
        "fld" => "Field director ",
        "flm" => "Film editor",
        "fmo" => "Former owner",
        "fpy" => "First party",
        "fnd" => "Funder",
        "frg" => "Forger",
        "gis" => "Geographic information specialist ",
        "grt" => "Graphic technician",
        "hnr" => "Honoree",
        "hst" => "Host",
        "ill" => "Illustrator",
        "ilu" => "Illuminator",
        "ins" => "Inscriber",
        "inv" => "Inventor",
        "itr" => "Instrumentalist",
        "ive" => "Interviewee",
        "ivr" => "Interviewer",
        "lbr" => "Laboratory ",
        "lbt" => "Librettist",
        "ldr" => "Laboratory director ",
        "led" => "Lead",
        "lee" => "Libelee-appellee",
        "lel" => "Libelee",
        "len" => "Lender",
        "let" => "Libelee-appellant",
        "lgd" => "Lighting designer",
        "lie" => "Libelant-appellee",
        "lil" => "Libelant",
        "lit" => "Libelant-appellant",
        "lsa" => "Landscape architect",
        "lse" => "Licensee",
        "lso" => "Licensor",
        "ltg" => "Lithographer",
        "lyr" => "Lyricist",
        "mcp" => "Music copyist",
        "mfr" => "Manufacturer",
        "mdc" => "Metadata contact",
        "mod" => "Moderator",
        "mon" => "Monitor",
        "mrk" => "Markup editor",
        "msd" => "Musical director",
        "mte" => "Metal-engraver",
        "mus" => "Musician",
        "nrt" => "Narrator",
        "opn" => "Opponent",
        "org" => "Originator",
        "orm" => "Organizer of meeting",
        "oth" => "Other",
        "own" => "Owner",
        "pat" => "Patron",
        "pbd" => "Publishing director",
        "pbl" => "Publisher",
        "pdr" => "Project director",
        "pfr" => "Proofreader",
        "pht" => "Photographer",
        "plt" => "Platemaker",
        "pma" => "Permitting agency",
        "pmn" => "Production manager",
        "pop" => "Printer of plates",
        "ppm" => "Papermaker",
        "ppt" => "Puppeteer",
        "prc" => "Process contact",
        "prd" => "Production personnel",
        "prf" => "Performer",
        "prg" => "Programmer",
        "prm" => "Printmaker",
        "pro" => "Producer",
        "prt" => "Printer",
        "pta" => "Patent applicant",
        "pte" => "Plaintiff -appellee",
        "ptf" => "Plaintiff",
        "pth" => "Patent holder",
        "ptt" => "Plaintiff-appellant",
        "rbr" => "Rubricator",
        "rce" => "Recording engineer",
        "rcp" => "Recipient",
        "red" => "Redactor",
        "ren" => "Renderer",
        "res" => "Researcher",
        "rev" => "Reviewer",
        "rps" => "Repository",
        "rpt" => "Reporter",
        "rpy" => "Responsible party",
        "rse" => "Respondent-appellee",
        "rsg" => "Restager",
        "rsp" => "Respondent",
        "rst" => "Respondent-appellant",
        "rth" => "Research team head",
        "rtm" => "Research team member",
        "sad" => "Scientific advisor",
        "sce" => "Scenarist",
        "scl" => "Sculptor",
        "scr" => "Scribe",
        "sds" => "Sound designer",
        "sec" => "Secretary",
        "sgn" => "Signer",
        "sht" => "Supporting host",
        "sng" => "Singer",
        "spk" => "Speaker",
        "spn" => "Sponsor",
        "spy" => "Second party",
        "srv" => "Surveyor",
        "std" => "Set designer",
        "stl" => "Storyteller",
        "stm" => "Stage manager",
        "stn" => "Standards body",
        "str" => "Stereotyper",
        "tcd" => "Technical director",
        "tch" => "Teacher",
        "ths" => "Thesis advisor",
        "trc" => "Transcriber",
        "trl" => "Translator",
        "tyd" => "Type designer",
        "tyg" => "Typographer",
        "vdg" => "Videographer",
        "voc" => "Vocalist",
        "wam" => "Writer of accompanying material",
        "wdc" => "Woodcutter",
        "wde" => "Wood -engraver",
        "wit" => "Witness"}
    end

    def valid_child_types
      ["data", "supporting file", "profile", "lorem ipsum", "dolor"]
    end

  end
  
  module InstanceMethods
  
    def build_solr(solr_doc)
      extract_person_full_names.each {|pfn| solr_doc << pfn }
      extract_uva_departments.each {|department| solr_doc << department}
      extract_person_organizations.each {|org| solr_doc << org }
      extract_person_full_names_and_computing_ids.each {|pfc| solr_doc << pfc }
      solr_doc << {:released_facet => "false"} if solr_doc["released_facet"].nil?
      solr_doc
    end
  
    # Remove the contributor entry identified by @contributor_type and @index
    def remove_contributor(contributor_type, index)
      self.find_by_terms( {contributor_type.to_sym => index.to_i} ).first.remove
      self.dirty = true
    end
  
    # Inserts a new contributor (mods:name) into the mods document
    # creates contributors of type :person, :organization, or :conference
    def insert_contributor(type, opts={})
      class_name = self.class
      case type.to_sym 
      when :person
        node = class_name.person_template
        nodeset = self.find_by_terms(:mods, :person)
      when :advisor
        node = class_name.advisor_template
        nodeset = self.find_by_terms('//oxns:name[@type="personal"][contains(oxns:role, "ths")]')
        person_nodeset = self.find_by_terms(:mods, :person)
        person_count = person_nodeset.length
      when :organization
        node = class_name.organization_template
        nodeset = self.find_by_terms(:organization)
      when :conference
        node = class_name.conference_template
        nodeset = self.find_by_terms(:conference)
      when :editor
        node = class_name.editor_template
        nodeset = self.find_by_terms(:book, :editor)
      else
        ActiveFedora.logger.warn("#{type} is not a valid argument for #{class_name}.insert_contributor")
        node = nil
        index = nil
      end
      
      unless nodeset.nil?
        if nodeset.empty?
          self.ng_xml.root.add_child(node)
          index = 0
        else
          if type.to_sym == :advisor
            person_nodeset.after(node)
            index = person_count
          else
            nodeset.after(node)
            index = nodeset.length
          end
        end
        self.dirty = true
      end
      
      return node, index
    end
  
  end
   
end